import { env } from "@/utils/env";
import * as schema from "@/server/db/schema";
import { drizzle } from "drizzle-orm/better-sqlite3";
import { drizzle as drizzleJson } from "drizzle-orm/json-db";
import fs from 'fs';
import path from 'path';

// Stellen sicher, dass DB_PATH tatsächlich gesetzt ist
const dbPath = env.DB_PATH || "/app/db/sqlite.db";
const jsonDbPath = env.DB_JSON_PATH || "/app/db/db.json";

// JSON-Fallback-Implementierung
class JsonDbAdapter {
  private data: Record<string, any[]> = {};
  private dbPath: string;

  constructor(dbPath: string) {
    this.dbPath = dbPath;
    this.loadFromDisk();
  }

  private loadFromDisk() {
    try {
      if (fs.existsSync(this.dbPath)) {
        const content = fs.readFileSync(this.dbPath, 'utf8');
        this.data = JSON.parse(content);
      } else {
        // Ensure directory exists
        const dir = path.dirname(this.dbPath);
        if (!fs.existsSync(dir)) {
          fs.mkdirSync(dir, { recursive: true });
        }
        this.saveToFile();
      }
    } catch (error) {
      console.error('Error loading JSON database:', error);
      this.data = {};
      this.saveToFile();
    }
  }

  private saveToFile() {
    try {
      fs.writeFileSync(this.dbPath, JSON.stringify(this.data, null, 2));
    } catch (error) {
      console.error('Error saving JSON database:', error);
    }
  }

  getTable(table: string) {
    if (!this.data[table]) {
      this.data[table] = [];
    }
    return {
      all: () => this.data[table],
      get: (id: string) => this.data[table].find(item => item.id === id),
      add: (item: any) => {
        this.data[table].push(item);
        this.saveToFile();
        return item;
      },
      update: (id: string, item: any) => {
        const index = this.data[table].findIndex(i => i.id === id);
        if (index >= 0) {
          this.data[table][index] = { ...this.data[table][index], ...item };
          this.saveToFile();
          return this.data[table][index];
        }
        return null;
      },
      delete: (id: string) => {
        const index = this.data[table].findIndex(i => i.id === id);
        if (index >= 0) {
          const deleted = this.data[table].splice(index, 1)[0];
          this.saveToFile();
          return deleted;
        }
        return null;
      }
    };
  }
}

// Versuche SQLite zu laden, mit Fallback auf JSON
let db;
try {
  // Versuche SQLite zu initialisieren
  console.log("Initialisiere SQLite-Datenbank...");
  const Database = require('better-sqlite3');
  
  // Konfiguriere SQLite für zuverlässigeren Betrieb
  const sqlite = new Database(dbPath, {
    // Setze längeres Timeout für Operationen auf langsamen Geräten (RPi)
    timeout: 30000,
    // Aktiviere WAL-Modus für höhere Performance
    journalMode: 'wal',
    // Verbesserte Fehlerbehandlung
    verbose: console.error,
  });

  // Aktiviere Fremdschlüssel-Constraints
  sqlite.pragma('foreign_keys = ON');

  // Exportiere die Drizzle-Datenbankinstanz
  db = drizzle(sqlite, { schema });
  console.log("SQLite-Datenbank erfolgreich initialisiert.");
} catch (error) {
  // Bei Fehler: Fallback auf JSON-Datenbank
  console.warn(`SQLite-Initialisierung fehlgeschlagen: ${error.message}`);
  console.warn("Verwende JSON-Fallback-Datenbank...");
  
  try {
    const jsonDbAdapter = new JsonDbAdapter(jsonDbPath);
    db = drizzleJson(jsonDbAdapter, { schema });
    console.log(`JSON-Datenbank wird verwendet: ${jsonDbPath}`);
  } catch (jsonError) {
    console.error("Konnte keine Datenbank initialisieren:", jsonError);
    throw new Error("Keine Datenbankverbindung möglich.");
  }
}

// Exportiere die Datenbankinstanz
export { db };
